#include <ansi_c.h>

/***********************************************************************
brl_gse.c

		peak detector gui/for BARREL programs  7May08/mm
		adapted from pd_disp.c

rev 1.0:	 19May08--first version
rev 1.1:	 19Jun08--added housekeeping decode/display
rev 1.2:	 24Jun08--rewrote serial acquisition routine
rev 1.3:	 21Jul08--200 byte frames; emits RCVD every 10 frames
rev 1.4:	 08Apr09--test flight version; no fluxgate; no RCVD; some
			button commands; 194 byte frames
rev 1.5		 30Sep09--fixed type problem with frame counter, added
			a noise diagnostic using the LL, HL, PD counters
			
************************************************************************

This program maintains a display of the returned peak detector data.

***********************************************************************/
#include <utility.h>
#include <rs232.h>
#include <stdio.h>
#include <time.h>
#include <string.h>
#include "gse.h"
#define FIFTYms (CLOCKS_PER_SEC * 0.050)
#define PORT 0
#define DFILE 1
#define FRM_LENGTH 194
#define SLOW_SPEC_SIZE 256
#define MED_SPEC_SIZE 48
#define LOST 0
#define EB 1
#define LOCK 2
#define PLUS 3
#define PP 4
#define PPM 5
typedef unsigned char BYTE;
typedef unsigned short WORD;
typedef struct
{
    short   swidth[SLOW_SPEC_SIZE];
    float   sx[SLOW_SPEC_SIZE];
    long    sy[SLOW_SPEC_SIZE];
    int     scnt[32];
    short   mwidth[48];
    float   mx[48];
    long    my[48];
    int     mcnt[4];
    float   cal;
} spec;
int     service(int, int, spec *);
int     get_port_frm(BYTE *);
int     get_file_frm(BYTE *);
int     resync();
short   glueshort(BYTE *);
long    gluelong(BYTE *);
void    hkpg(int, WORD, unsigned int);
void    init(spec *);
void    unpack(BYTE *, int *, int);
void    frm_process(BYTE *, spec *, int, int);
static FILE *inf,
       *ouf;
static long garbage = 0;


/**********************************************************************
			Start of main function.
**********************************************************************/
main(int argc, char *argv[])
{
    int     quit,
	    src;
    int     mode,
	    panel,
	    status,
	    menu;
    static spec spectrum;
    static struct tm *st;
    static time_t x;
    static char filename[13];


	
	
    if (argc == 1) {
	src = PORT;
	quit = OpenComConfig(1, "COM1", 19200, 0, 8, 1, 0, 0);
	ComSetEscape(1,CLRDTR);
	x=time(NULL);
	st=localtime(&x);
	sprintf(filename,"%1d%03d%02d%02d.dat",st->tm_year%10,
	  st->tm_yday,st->tm_hour,st->tm_min);
	ouf = fopen(filename, "wb");
	if (quit < 0) {
	    printf("%d COM1 port error %d\n", quit);
	    return 0;
	}
    }

    else if (argc == 2) {
	if ((inf = fopen(argv[1], "rb")) == NULL) {
	    printf("Cannot open input file %s\n", argv[1]);
	    return 0;
	}
	src = DFILE;
    }
    quit = 0;
    init(&spectrum);
    panel = LoadPanel(0, "gse.uir", BRL);
    SetCtrlAttribute(panel,BRL_TERM,ATTR_ON_COLOR,VAL_YELLOW);
    status = DisplayPanel(panel);
    while (!quit) {
	quit = service(panel, src, &spectrum);
    }
    DiscardPanel(panel);
    if (src == PORT)
	CloseCom(1);

    else
	fclose(inf);
    fclose(ouf);
    return 0;
}  /* end of main function */

/**********************************************************************
    This function issues a cutdown sequence
**********************************************************************/
//int CVICALLBACK 
//cutdown(int panel, int control, int event, void *callbackData, int eventData1,
//int eventData2)
//{     
//	int temp;
//	if (event==EVENT_LEFT_CLICK) {
//		GetCtrlAttribute(panel,control, ATTR_CMD_BUTTON_COLOR, &temp); 
//		ComWrt(1,"A\r",2);
//		SetCtrlAttribute(panel,control, ATTR_CMD_BUTTON_COLOR,VAL_RED);
//		Delay(3.0);
//		ComWrt(1,"K\r",2);
//		SetCtrlAttribute(panel,control, ATTR_CMD_BUTTON_COLOR,temp); 
//	}
//	return (0);
//}

/**********************************************************************
    This function services the display
**********************************************************************/
int
service(int panel, int src, spec * p)
{
    static int i,
	    j,
	    quit,
	    mode,
	    status,
	    err_code,
	    isevent,
	    con,
	    pan;
    static BYTE hex[521];
    static int tm_state = 0;
    static int xmin,
	    xmax,
	    offset;
    static int temp,
	    byte_cnt;
    static long acq_cntr,
	    cntr;
    static float ymin,
	    ymax,
	    mdiffspec[MED_SPEC_SIZE],
	    diffspec[SLOW_SPEC_SIZE];
    static BYTE inbuf[FRM_LENGTH];
    static char msg[30];

    quit = 0;

/*
	This positions us at the end of the last incoming serial block
	For 19200 baud and FRM_LENGTH byte frames, data comes in 101ms bursts
	followed by 899ms of quiet.
	First require 100ms period of quiet, which means it is now
	after the last record has come in. Then extend the timeout
	to 1+ seconds.
*/
    if (src == PORT) {
	SetCtrlVal(panel, BRL_STATUS, "setting up serial port");
	SetComTime(1, 0.1);
	while (ComRdByte(1) >= 0);
	SetComTime(1, 1.05);
	acq_cntr = 1000;
	SetCtrlVal(panel, BRL_STATUS, "serial port ready");
    }

    else
	acq_cntr = 10000;
    xmin = 0;
    xmax = 4095;
    ymin = 1.e-3;
    ymax = 10.0;
    mode = 1;
    for (i = 0; i < SLOW_SPEC_SIZE; i++)
	p->sy[i] = 0.0;
    while (!quit) {
	if ((isevent = GetUserEvent(0, &pan, &con))) {
	    switch (con) {
	    case BRL_ACQ:
		GetCtrlVal(pan, con, &mode);
		if (!mode)
		    SetCtrlAttribute(pan, BRL_CNT, 10, 2);
		break;
	    case BRL_CNT:
		GetCtrlVal(pan, con, &acq_cntr);
		break;
	    case BRL_CLR:
		for (i = 0; i < MED_SPEC_SIZE; i++)
		    p->my[i] = 0;
		for (i = 0; i < 4; i++)
		    p->mcnt[i] = 0;
		DeleteGraphPlot(panel, BRL_MSPEC, -1, 1);
		for (i = 0; i < SLOW_SPEC_SIZE; i++)
		    p->sy[i] = 0;
		for (i = 0; i < 32; i++)
		    p->scnt[i] = 0;
		DeleteGraphPlot(panel, BRL_SPEC, -1, 1);
		break;
	    case BRL_XLOGLIN:
		GetCtrlVal(pan, con, &temp);
		SetCtrlAttribute(pan, BRL_SPEC, ATTR_XMAP_MODE,
		    temp ? VAL_LOG : VAL_LINEAR);
		break;
	    case BRL_YLOGLIN:
		GetCtrlVal(pan, con, &temp);
		if (temp == VAL_LOG && ymin < 1e-4) {
		    ymin = 1e-4;
		    SetAxisRange(pan, BRL_SPEC, VAL_NO_CHANGE, xmin, xmax,
			VAL_MANUAL, ymin, ymax);
		}
		SetCtrlAttribute(pan, BRL_SPEC, ATTR_YMAP_MODE,
		    temp ? VAL_LOG : VAL_LINEAR);
		break;
	    case BRL_XMIN:
		GetCtrlVal(pan, con, &xmin);
		SetAxisRange(pan, BRL_SPEC, VAL_MANUAL, xmin, xmax,
		    VAL_NO_CHANGE, ymin, ymax);
		break;
	    case BRL_XMAX:
		GetCtrlVal(pan, con, &xmax);
		SetAxisRange(pan, BRL_SPEC, VAL_MANUAL, xmin, xmax,
		    VAL_NO_CHANGE, ymin, ymax);
		break;
	    case BRL_YMIN:
		GetCtrlVal(pan, con, &ymin);
		SetAxisRange(pan, BRL_SPEC, VAL_NO_CHANGE, xmin, xmax,
		    VAL_MANUAL, ymin, ymax);
		break;
	    case BRL_YMAX:
		GetCtrlVal(pan, con, &ymax);
		SetAxisRange(pan, BRL_SPEC, VAL_NO_CHANGE, xmin, xmax,
		    VAL_MANUAL, ymin, ymax);
		break;
	    case BRL_TM_SWITCH:
		if (src != PORT)
		    break;
		GetCtrlVal(pan, con, &tm_state);
		if (tm_state) {
		    ComSetEscape(1, SETDTR);
		} else {
		    ComSetEscape(1, CLRDTR); 
		}
		break;
	    case BRL_ARM:
		if (src == PORT) {
		    ComWrtByte(1, 'A');
		    ComWrtByte(1, '\r');
		}
		SetCtrlVal(pan,con,0);
		break;
	    case BRL_MDM_RESET:
		if (src == PORT) {
		    ComWrtByte(1, 'M');
		    ComWrtByte(1, '\r');
		}
		SetCtrlVal(pan,con,0);
		break;
	    case BRL_DIS:
		if (src == PORT) {
		    ComWrtByte(1, 'D');
		    ComWrtByte(1, '\r');
		}
		SetCtrlVal(pan,con,0); 
		break;
	    case BRL_CUT:
		if (src == PORT) {
		    ComWrtByte(1, 'K');
		    ComWrtByte(1, '\r');
		}
		SetCtrlVal(pan,con,0); 
		break;
	    case BRL_NOOP:
		if (src == PORT) {
		    ComWrtByte(1, 'N');
		    ComWrtByte(1, '\r');
		}
		SetCtrlVal(pan,con,0); 
		break;
	    case BRL_QUIT:
		if (src == PORT) {
		    ComSetEscape(1, CLRDTR);
		}
		return (1);
		break;
	    default:
		return 0;
	    }
	    continue;
	}
	if (src == PORT)
	    err_code = get_port_frm(inbuf);
	else
	    err_code = get_file_frm(inbuf);

	if (err_code) {
	    if (src == PORT)
		switch (err_code) {
		case -1:
		    sprintf(msg, "COM1 err: too few bytes read");
		    break;
		case -2:
		    sprintf(msg, "COM1 err: illegal ++- sequence");
		    break;
		case -3:
		    sprintf(msg, "COM1 err: invalid state");
		    break;
		default:
		    sprintf(msg, "COM1 err: checksum failure");
		    break;
	    } else
		sprintf(msg, "EOF on input file");
	}

	else {
	    msg[0] = ' ';
	    msg[1] = '\0';
	    frm_process(inbuf, p, panel, mode);
	    DeleteGraphPlot(panel, BRL_MSPEC, -1, 1);
	    for (i = 0; i < MED_SPEC_SIZE; i++) {
		cntr = p->mcnt[i / 12];
		if (cntr)
		    mdiffspec[i] =
			p->my[i] / (4 * p->mwidth[i] * cntr * p->cal);

		else
		    mdiffspec[i] = 0;
	    }
	    PlotXY(panel, BRL_MSPEC, p->mx, mdiffspec, MED_SPEC_SIZE,
		VAL_FLOAT, VAL_FLOAT, VAL_THIN_STEP, 0,
		VAL_SOLID, 1, VAL_BLUE);
	    if (mode)
		if (acq_cntr != 0) {
		    DeleteGraphPlot(panel, BRL_SPEC, -1, 1);
		    for (i = 0; i < SLOW_SPEC_SIZE; i++) {
			cntr = p->scnt[i / 8];
			if (cntr)
			    diffspec[i] =
				p->sy[i] / (32 * p->swidth[i] * cntr *
				p->cal);

			else
			    diffspec[i] = 0;
		    }
		    PlotXY(panel, BRL_SPEC, p->sx, diffspec, SLOW_SPEC_SIZE,
			VAL_FLOAT, VAL_FLOAT, VAL_THIN_STEP, 0,
			VAL_SOLID, 1, VAL_BLUE);
		    acq_cntr--;
		    SetCtrlVal(panel, BRL_CNT, acq_cntr);
		} else {
		    SetCtrlAttribute(panel, BRL_CNT, 10, 2);
		    mode = 0;
		    SetCtrlVal(panel, BRL_ACQ, 0);
		}
	    for (i = 0, j = 0; i < FRM_LENGTH; i++)
		if (i % 32 == 0) {
		    sprintf(hex + j, "\n%02X", inbuf[i]);
		    j += 3;
		} else {
		    sprintf(hex + j, "%02X", inbuf[i]);
		    j += 2;
		}
	    hex[j++] = 10;
	    hex[j] = '\0';
	    SetCtrlVal(panel, BRL_RAW, hex);
	    SetCtrlVal(panel, BRL_JUNK, garbage);
	}
	SetCtrlVal(panel, BRL_STATUS, msg);
    }
    return 0;
}/******* END service() ********/

 /* Back up FRM_LENGTH-1 bytes in input file and search for sync word
	returns -1 on failure; 0 on success
*/
int
resync()
{
    static WORD temp,
	    state;
    static BYTE x;

    state = 0;
    Delay(1.0);
    fseek(inf, 1L - FRM_LENGTH, SEEK_CUR);
    garbage++;
    while ((temp = getc(inf)) != EOF) {
	garbage++;
	x = temp;
	switch (state) {
	case 0:
	    if (x == 0xEB)
		state = 1;
	    break;
	case 1:
	    if (x == 0x90) {
		fseek(inf, -2L, SEEK_CUR);
		garbage -= 2;
		return 0;
	    }
	    state = 0;

	    break;
	default:
	    state = 0;
	    break;
	}
    }
    return 1;
}


/* get_port_frm acquires a frame of data from the serial port
	INPUT p-> FRM_LENGTH byte buffer into which a frame is written
		and we read data from COM1 port at 19200 baud
	OUTPUT: buffer will be filled with a frame if successful
		return codes are as follows,
		0 if data was retrieved
		-1 if timeout on serial port
		-2 not used
		-3 is an invalid state trap for state machine
		-4 is a checksum failure
		we also write every byte to an output file (ouf)
	CALLS: none
	IMPLEMENTATION: state machine to read data until we have
		accumulated a frame;
		frames must start with 0xEB 0x90, and checksum must
		be valid. No byte for > 1 second is a timeout
		condition
	SIDE EFFECTS: increments garbage counter.

*/
int
get_port_frm(BYTE * p)
{
    static int byte_cnt,state;
    static BYTE *q, byte;
    static short val;
    static WORD x,
	    i,
	    temp,
	    chksum;

    state = LOST;
    q = p;
    while ((val = ComRdByte(1)) >= 0) {
	byte = (BYTE) val;
	fwrite(&byte, 1, 1, ouf);

	switch (state) {
	case LOST:
	    if (byte == 0xEB) {
		*q++ = byte;
		state = EB;
	    } else
		garbage++;
	    break;
	case EB:
	    if (byte = 0x90) {
		*q++ = byte;
		state = LOCK;
	    } else {
		q = p;
		garbage++;
		state = LOST;
	    }
	    break;
	case LOCK:
	    *q++ = byte;
	    break;
	default:
	    return -3;
	}
	if ((q - p) >= FRM_LENGTH) {
	    chksum = glueshort(p + FRM_LENGTH - 2);
	    x = 0;
	    for (i = 0; i < FRM_LENGTH - 2; i += 2) {
		temp = glueshort(p + i);
		x += temp;
	    }
	    if (x == chksum)
		return 0;
	    garbage += (q-p);
	    return -4;
	}
    }
    return -1;
}

/* get_file_frm acquires a frame of data from a file
	INPUT p-> FRM_LENGTH byte buffer into which a frame is written
	OUTPUT	0 if data was retrieved
		-1 if timeout on EOF

	buffer will be filled with a frame if successful
	wait for 100ms between frames,
	corresponding to a readback 10X faster than real time
*/
int
get_file_frm(BYTE * p)
{
    static int byte_cnt;
    static unsigned short x,
	    i,
	    temp,
	    chksum;

    Delay(0.1);
    while (fread(p, FRM_LENGTH, 1, inf) == 1) {
	chksum = glueshort(p + FRM_LENGTH - 2);
	x = 0;
	for (i = 0; i < FRM_LENGTH - 2; i += 2) {
	    temp = glueshort(p + i);
	    x += temp;
	}
	if (x == chksum)
	    return 0;
	if (resync() )
	    return -1;
    }
    return -1;
}

void
  init(spec * p)
{
    static int i,
	    subtotal;
    static short widths[MED_SPEC_SIZE] =
	{ 4, 4, 3, 4, 3, 4, 6, 8, 6, 8, 8, 6, 8, 6, 8, 12, 16, 12, 16, 16, 12,
	16, 12, 16, 24, 32, 24, 32, 32, 24, 32, 24, 32, 48, 64, 48, 64,
	64, 48, 64, 48, 64, 96, 128, 96, 128, 128, 96
    };

    for (i = 0; i < 64; i++)
	p->swidth[i] = 1;
    for (i = 64; i < 96; i++)
	p->swidth[i] = 2;
    for (i = 96; i < 128; i++)
	p->swidth[i] = 4;
    for (i = 128; i < 160; i++)
	p->swidth[i] = 8;
    for (i = 160; i < 192; i++)
	p->swidth[i] = 16;
    for (i = 192; i < 224; i++)
	p->swidth[i] = 32;
    for (i = 224; i < SLOW_SPEC_SIZE; i++)
	p->swidth[i] = 64;
    p->sx[0] = 0;
    for (i = 1; i < SLOW_SPEC_SIZE; i++)
	p->sx[i] = p->sx[i - 1] + p->swidth[i];
    for (i = 0; i < SLOW_SPEC_SIZE; i++)
	p->sx[i] += p->swidth[i] / 2.0;
    for (i = 0; i < 32; i++)
	p->scnt[i] = 0;

    subtotal = p->mx[0] = 42;
    for (i = 0; i < MED_SPEC_SIZE; i++) {
	p->mwidth[i] = widths[i];
	subtotal += widths[i];
	p->mx[i] = subtotal - widths[i] / 2;
    }
    p->mcnt[0] = p->mcnt[1] = p->mcnt[2] = p->mcnt[3] = 0;
    p->cal = 2.2;	/* 2.2 keV/bin */
    return;
}

void
unpack(BYTE * p, int *q, int cnt)
{
    static unsigned short temp;

    while (cnt > 0) {
	temp = (*p++) << 4;
	temp |= ((*p) >> 4);
	*q++ = temp;
	temp = ((*p++) << 8) & 0xFFF;
	temp |= *p++;
	*q++ = temp;
	cnt -= 2;
    }
    return;
}

void
frm_process(BYTE * p, spec * sp, int pan, int mode)
{
    static char timestr[13];
    static int i,
	    j,
	    q[52],
	    temp,
	    dow,
	    hh,
	    mm;
    static int fcmod4,
	    fcmod32;
    static WORD lc[80];
    static short stemp,ll=0,irq=0,hl=0,pd=0;
    static long ltemp;
    static unsigned int fc;
    static float ftemp;
    static double dtemp;

    stemp = glueshort(p + 2);
    SetCtrlVal(pan, BRL_VER, (BYTE) (stemp >> 11));
    SetCtrlVal(pan, BRL_ID, (BYTE) ((stemp >> 5) & 0x3F));
    fcmod4 = *(p + 5) & 0x3;
    fcmod32 = *(p + 5) & 0x1f;
    fc = ((*(p + 3) & 0x1F) << 16) + (WORD) glueshort(p + 4);
    
    ltemp = gluelong(p + 12);
    ftemp = *((float *) &ltemp);
    SetCtrlVal(pan, BRL_MAGX, ftemp);
    ltemp = gluelong(p + 16);
    ftemp = *((float *) &ltemp);
    SetCtrlVal(pan, BRL_MAGY, ftemp);
    ltemp = gluelong(p + 20);
    ftemp = *((float *) &ltemp);
    SetCtrlVal(pan, BRL_MAGZ, ftemp);
    ltemp = gluelong(p + 24);
    ftemp = *((float *) &ltemp);
    temp = glueshort(p + 190);
    ltemp = gluelong(p + 6);

    switch (fcmod4) {
    case 0:
	SetCtrlVal(pan,BRL_DIFF,ll-(pd+hl));
	SetCtrlVal(pan, BRL_PITCH, ftemp);
	SetCtrlVal(pan, BRL_IRQ, temp);
	dtemp = ltemp / 1000.;
	SetCtrlVal(pan, BRL_ALT, dtemp);
	break;
    case 1:
	SetCtrlVal(pan, BRL_ROLL, ftemp);
	SetCtrlVal(pan, BRL_LL, temp);
	ll = temp;
	dtemp = ltemp / 1000.;
	dow = (int) (dtemp / 86400.);
	dtemp -= 86400. * dow;
	hh = dtemp / 3.6e3;
	dtemp -= 3.6e3 * hh;
	mm = dtemp / 6.e1;
	dtemp -= 6.0e1 * mm;
	sprintf(timestr, "%1d/%02d:%02d:%04.1f", dow, hh, mm, dtemp);
	SetCtrlVal(pan, BRL_TIME, timestr);
	break;
    case 2:
	SetCtrlVal(pan, BRL_PITCH, ftemp);
	SetCtrlVal(pan, BRL_PD, temp);
	dtemp = ltemp * 8.38190e-8;
	SetCtrlVal(pan, BRL_LAT, dtemp);
	pd = temp;
	break;
    case 3:
	SetCtrlVal(pan, BRL_ROLL, ftemp);
	SetCtrlVal(pan, BRL_HL, temp);
	dtemp = ltemp * 8.38190e-8;
	SetCtrlVal(pan, BRL_LON, dtemp);
	hl = temp;
	break;
    default:
	break;
    }


    temp = glueshort(p + 10);
    SetCtrlVal(pan, BRL_PPS, temp);
    hkpg(pan, glueshort(p + 28), fc);

    if (mode) {
	temp = 8 * fcmod32;
	sp->scnt[fcmod32]++;
	if (mode > 0)
	    for (i = 0, j = 174; i < 8; i++, j += 2)
		sp->sy[i + temp] += glueshort(p + j);
	else
	    for (i = 0, j = 174; i < 8; i++, j += 2)
		sp->sy[i + temp] -= glueshort(p + j);
    }

    temp = 12 * fcmod4;
    sp->mcnt[fcmod4]++;
    for (i = 0, j = 150; i < 12; i++, j += 2)
	sp->my[i + temp] += glueshort(p + j);

    for (i = 30, j = 0; i < 150; i += 6) {
	lc[j++] = glueshort(p + i);
	lc[j++] = glueshort(p + i + 2);
	lc[j++] = *(p + i + 4);
	lc[j++] = *(p + i + 5);
    }
    PlotStripChart(pan, BRL_LC, lc, 80, 0, 0, VAL_UNSIGNED_SHORT_INTEGER);
    return;
}

/* interpret and display the housekeeping word; depends on frame cntr */
void
hkpg(int pan, WORD data, unsigned int framecntr)
{
    static float temp;

    switch (framecntr % 40) {
    case 0:
	SetCtrlVal(pan, BRL_V0, 20. * data / 6.5535e4);
	break;
    case 1:
	SetCtrlVal(pan, BRL_V1, 20. * data / 6.5535e4);
	break;
    case 2:
	SetCtrlVal(pan, BRL_V2, 40. * data / 6.5535e4);
	break;
    case 3:
	SetCtrlVal(pan, BRL_V3, 10. * data / 6.5535e4);
	break;
    case 4:
	SetCtrlVal(pan, BRL_V4, 10. * data / 6.5535e4);
	break;
    case 5:
	SetCtrlVal(pan, BRL_V5, 10. * data / 6.5535e4);
	break;
    case 6:
	SetCtrlVal(pan, BRL_V6, -10. * data / 6.5535e4);
	break;
    case 7:
	SetCtrlVal(pan, BRL_V7, -10. * data / 6.5535e4);
	break;
    case 8:
	SetCtrlVal(pan, BRL_I0, 3.33 * data / 6.5535e4);
	break;
    case 9:
	SetCtrlVal(pan, BRL_I1, 3.33 * data / 6.5535e4);
	break;
    case 10:
	SetCtrlVal(pan, BRL_I2, 3.33 * data / 6.5535e4);
	break;
    case 11:
	SetCtrlVal(pan, BRL_I3, 0.667 * data / 6.5535e4);
	break;
    case 12:
	SetCtrlVal(pan, BRL_I4, 0.100 * data / 6.5535e4);
	break;
    case 13:
	SetCtrlVal(pan, BRL_I5, 3.33 * data / 6.5535e4);
	break;
    case 14:
	SetCtrlVal(pan, BRL_I6, 0.0077 * data / 6.5535e4);
	break;
    case 15:
	SetCtrlVal(pan, BRL_I7, 0.100 * data / 6.5535e4);
	break;
    case 16:
	SetCtrlVal(pan, BRL_T0, 500. * data / 65535.0 - 273.15);
	break;
    case 17:
	SetCtrlVal(pan, BRL_T1, 500. * data / 6.5535e4 - 273.15);
	break;
    case 18:
	SetCtrlVal(pan, BRL_T2, 500. * data / 6.5535e4 - 273.15);
	break;
    case 19:
	SetCtrlVal(pan, BRL_T3, 500. * data / 6.5535e4 - 273.15);
	break;
    case 20:
	SetCtrlVal(pan, BRL_T4, 500. * data / 6.5535e4 - 273.15);
	break;
    case 21:
	SetCtrlVal(pan, BRL_T5, 500. * data / 6.5535e4 - 273.15);
	break;
    case 22:
	SetCtrlVal(pan, BRL_T6, 500. * data / 6.5535e4 - 273.15);
	break;
    case 23:
	SetCtrlVal(pan, BRL_T7, 500. * data / 6.5535e4 - 273.15);
	break;
    case 24:
	SetCtrlVal(pan, BRL_T8, 500. * data / 6.5535e4 - 273.15);
	break;
    case 25:
	SetCtrlVal(pan, BRL_T9, 500. * data / 6.5535e4 - 273.15);
	break;
    case 26:
	SetCtrlVal(pan, BRL_T10, 500. * data / 6.5535e4 - 273.15);
	break;
    case 27:
	SetCtrlVal(pan, BRL_T11, 500. * data / 6.5535e4 - 273.15);
	break;
    case 28:
	SetCtrlVal(pan, BRL_T12, 500. * data / 6.5535e4 - 273.15);
	break;
    case 29:
	SetCtrlVal(pan, BRL_T13, 40. * data / 6.5535e4);
	break;
    case 30:
	SetCtrlVal(pan, BRL_T14, 40. * data / 6.5535e4);
	break;
    case 31:
	SetCtrlVal(pan, BRL_T15, 5. * data / 6.5535e4);
	break;
    case 32:
	SetCtrlVal(pan, BRL_V8, 10. * data / 6.5535e4);
	break;
    case 33:
	SetCtrlVal(pan, BRL_V9, 40. * data / 6.5535e4);
	break;
    case 34:
	SetCtrlVal(pan, BRL_V10, 40. * data / 6.5535e4);
	break;
    case 35:
	SetCtrlVal(pan, BRL_V11, 40. * data / 6.5535e4);
	break;
    case 36:
	SetCtrlVal(pan, BRL_SATS, (BYTE) (data >> 8));
	SetCtrlVal(pan, BRL_OFFSET, (BYTE) (data & 0xff));
	break;
    case 37:
	SetCtrlVal(pan, BRL_WEEK, data);
	break;
    case 38:
	SetCtrlVal(pan, BRL_CMDS, data & 0x7FFF);
	if (data & 0x8000) SetCtrlVal(pan, BRL_TERM, 0);
	else SetCtrlVal(pan,BRL_TERM,1);
	break;
    case 39:
	SetCtrlVal(pan, BRL_MODEM, data & 0xFF);
	SetCtrlVal(pan, BRL_DCD, data >> 8); 
	break;
    default:
	break;
    }
    if ((framecntr % 40) == 36) SetCtrlVal(pan, BRL_TEMP, data); 
    return;
}


/* join 2 consecutive bytes into an INT16 */
short
glueshort(BYTE * p)
{
    static short x;

    x = (*p << 8) | *(p + 1);
    return x;
}


/* join 4 consecutive bytes into an INT32 */
long
gluelong(BYTE * p)
{
    static long x;

    x = (*p << 8) | *(p + 1);
    x = (x << 8) | *(p + 2);
    x = (x << 8) | *(p + 3);
    return x;
}
